package com.infinite.utils.checksum;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.zip.CRC32;

/**
 * Klasa stanowiąca pomoc w generowaniu sum kontrolnychdla strumienia wejsciowego.
 * Obecnie obslugiwane algorytmy sum kontrolnych: CRC32, MD5, SHA1.
 * CRC32 - cechuje sie duza popularnoscia i szybkoscia ale stosunkowo latwo jest trafic na rozne dane o tej samej sumie kontrolnej wyznaczonej tym algorytmem
 * MD5 i SHA1 daja bardzo niskie prawdopodpbienstwo powtarzalnosci, lecz wymagaja wiekszej mocy obliczeniowej (w MD5 odkryto pewna regularnosc, zaleca sie uzywanie SHA1)
 */
public class ChecksumHelper {

  /**
   * Obliczanie sumy kontrolnej wg zadanego algorytmu dla zadanego strumienia.
   * @param is Strumien danych, dla ktorych nalezy wyliczyc sume kontrolna.
   * @param alg Algorytm wyliczania sumy kontrolnej (CRC32, MD5, SHA1)
   * @return Suma kontrolna w postaci ciagu znakow, w przypadku MD5 i SHA1 jest to szesnastkowy zapis duzej liczby calkowitej
   * @throws IOException Blad czytania ze strumienia danych
   * @throws NoSuchAlgorithmException Nieprawidlowy algorytm wyznaczania sumy kontrolnej
   */
  public static String computeChecksum(InputStream is, String alg)
  throws IOException, NoSuchAlgorithmException {
    String cs = "";
    if (alg.equals("CRC32")) {
      cs = computeCRC32(is)+"";
    }
    else {
      byte[] bytes = computeMessageDigest(is, alg);
      cs = new BigInteger(1, bytes).toString(16).toUpperCase();
    }
    return cs;
  }

  /**
   * Obliczanie sumy kontrolnej CRC32 dla zadanego strumienia.
   * @param is Strumien danych, dla ktorych nalezy wyliczyc sume kontrolna.
   * @return Tekstowy zapis liczby calkowitej
   * @throws IOException Blad czytania ze strumienia danych
   */
  public static long computeCRC32(InputStream is)
  throws IOException {
    CRC32 checksum = new CRC32();
    byte[] buffer = new byte[1024];
    int len = 0;
    while ((len = is.read(buffer)) >= 0) {
      checksum.update(buffer, 0, len);
    }
    return checksum.getValue();
  }


  /**
   * Obliczanie sumy kontrolnej wg zadanego algorytmu dla zadanego strumienia.
   * @param is Strumien danych, dla ktorych nalezy wyliczyc sume kontrolna.
   * @param alg Algorytm wyliczania sumy kontrolnej (MD5, SHA1)
   * @return Suma kontrolna w postaci ciagu znakow stanowiacego szesnastkowy zapis duzej liczby calkowitej
   * @throws IOException Blad czytania ze strumienia danych
   * @throws NoSuchAlgorithmException Nieprawidlowy algorytm wyznaczania sumy kontrolnej
   */
  public static byte[] computeMessageDigest(InputStream is, String alg)
  throws NoSuchAlgorithmException, IOException {
    MessageDigest md = MessageDigest.getInstance(alg);
    byte[] buffer = new byte[1024];
    int len = 0;
    while ((len = is.read(buffer)) >= 0) {
      md.update(buffer, 0, len);
    }
     return md.digest();
  }  
  
}
